//===========================================================================
//
//  Filename  : gantt.cpp
//  Purpose   : Gantt View for QtGantt
//  Copyright : Copyright (C) 2000 Matt J. Gumbley <matt@gumbley.demon.co.uk>
//  Original  : by Gunnstein Lye
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//===========================================================================

#include <iostream.h>

#include <qpainter.h>
#include <qprinter.h>
#include <qprintdialog.h>
#include <qmessagebox.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qstring.h>
#include <qdatetime.h>
#include <qevent.h>

#include "task.h"
#include "project.h"
#include "gantt.h"


Gantt::Gantt( Project *proj, QWidget *parent, const char *name )
        : QWidget( parent, name )
	{
	project = proj;
	setBackgroundColor( "white" );
	textpixx = 150;
	daypix = 10;
	isprinting = 0;
	weekdetail = 0;
	TEXTPIXY = 50;
	}




void Gantt::print( QPrinter *printer )  // draw gantt to printer
	{
	this->printer = printer;
	if( project->numtasks > 0 )    // if file reading went OK
		{
		if ( printer->setup() )   // if pushed OK button in printer setup
			{
			isprinting = 1;
			QPainter painter;
			painter.begin( printer );
			paint( &painter );      // calls the paint function
			painter.end();
			isprinting = 0;
			}
		}
	this->printer = 0;
	}

// draw gantt to file without setup dialog
void Gantt::printToFile( QPrinter * printer, char * filename )
	{
	this->printer = printer;
	if( project->numtasks > 0 )    // if file reading went OK
		{
			isprinting = 1;
			QPainter painter;
			printer->setOutputFileName( filename );
			painter.begin( printer );
			paint( &painter );      // calls the paint function
			painter.end();
			isprinting = 0;
		}
	this->printer = 0;
	}


void Gantt::draw()  // draw gantt to screen
	{
	if( project->numtasks > 0 )   // if file reading went OK
		{
		QPainter painter;
		painter.begin( this );
		paint( &painter );
		painter.end();
		}
	}


void Gantt::changetextpix( int tpix )
	{
	textpixx = tpix;
	repaint();
	}


void Gantt::changedaypix( int dpix )
	{
	daypix = dpix;
	repaint();
	}

void Gantt::setweekdetail( int newweekdetail )
	{
	weekdetail = newweekdetail;
	repaint();
	}


void Gantt::paint( QPainter *painter )  // draw gantt, used by print() and draw()
	{
	int page=1, pagewidth, pageheight;
	int textHeight, ascent, twoDigitWidth, leading;
	int canFitDayNums = 0;
	if( isprinting )
		{
//printf("*** PRINTING ***\n");
		// CHANGE: this ain't gonna work - we need to know exactly how
		// big our pages are, so we can calculate pagetasks (before main
		// loop). The Qt book has something about printing.
		pagewidth = 1200;
		pageheight = 800;
		}
	else
		{
//printf("*** DISPLAYING ***\n");
		pagewidth = width();
		pageheight = height();
		}
	
	// Draws the header text
	painter->setPen( "black" );
	painter->setFont( QFont("times", 20, QFont::Bold) );
	painter->drawText( 10, 0, pagewidth-20, TITLEPIXY, AlignCenter, 
				project->name->copy() );
	QFontMetrics qfm = painter->fontMetrics();
	ascent = qfm.ascent();
	textHeight = ascent + qfm.descent();

	// Draws lines for weeks, months and current date
	painter->setFont( QFont("helvetica", FONTSIZE, QFont::Normal) );
	qfm = painter->fontMetrics();
	ascent = qfm.ascent();
	textHeight = ascent + qfm.descent();
	twoDigitWidth = qfm.width("33", 2);
	leading = qfm.leading();

	// If we're doing week detail, the calendar header is taller, to take
	// into account the start-of-week day numbers (but only if they'll fit)
	if ( weekdetail && daypix > (twoDigitWidth+8) )
		{
		TEXTPIXY = TITLEPIXY + 10 + (textHeight * 2);
		canFitDayNums = 1;
		}
	else
		{
		TEXTPIXY = TITLEPIXY + 10 + textHeight;
		canFitDayNums = 0;
		}


	int i=0, pagetasks=20, currentpage;
	pagetasks = (pageheight-TEXTPIXY) / SPACING;
//printf("pageheight is %d, pagetasks is %d\n", pageheight, pagetasks);
	while( i<project->numtasks )
		{
		currentpage = page;

//printf("Drawing headings...\n");
		// Draw top border around months and optional start-of-week day
		// numbers
		painter->setPen( "darkgrey" );
		painter->drawLine( 0, TITLEPIXY, pagewidth, TITLEPIXY );
		painter->drawLine( 0, TEXTPIXY, pagewidth, TEXTPIXY );

		// Draws lines for weeks, months and current date
		painter->setFont( QFont("helvetica", FONTSIZE, QFont::Normal) );
		//QFontMetrics qfm = painter->fontMetrics();
		//ascent = qfm.ascent();
		//textHeight = ascent + qfm.descent();
		QDate tempdate = project->startdate->addDays(0);
		for( int day=textpixx; day < pagewidth; day = day+daypix )
			{
			// fill in weekends with light grey
			if( weekdetail && 
			    (tempdate.dayOfWeek() == 6 || 
			    tempdate.dayOfWeek() == 7))
				{
				painter->setPen ( "lightgrey" );
				painter->setBrush ( "lightgrey" );
				painter->drawRect( day, TEXTPIXY+1, daypix, pageheight);
				}
			// indicate Mondays with a grey line
			if( weekdetail || tempdate.dayOfWeek() == 1 )  // if a monday
				{
				QPen dottedpen( weekdetail ? "black" : "lightgrey", 0, DashLine );
				painter->setPen ( dottedpen );
				painter->drawLine( day, TEXTPIXY, day, pageheight );  // Draws a dotted line for each week
				// display the day number for the start of each
				// week
				if ( canFitDayNums && tempdate.dayOfWeek() == 1 ) // weekdetail is implied by canFitDayNums
					{
					painter->setPen( "black" );
					QString qstr;
					int dayNoY = TITLEPIXY + 5 + textHeight + ascent + leading;
					painter->drawText( day+5, dayNoY, qstr.setNum(tempdate.day()));
					}
				}
			if( tempdate.day() == 1 )    // if first day of month
				{
				painter->setPen( "darkgrey" );
				painter->drawLine( day, TITLEPIXY, day, pageheight );  // Draws a solid line for each month
				painter->setPen( "black" );
				int monthY = TITLEPIXY + 5 + ascent;
				switch( tempdate.month() )
					{
					// these drawTexts are baseline-relative.
					case 1: painter->drawText( day+5, monthY, "January"); break;
					case 2: painter->drawText( day+5, monthY, "February"); break;
					case 3: painter->drawText( day+5, monthY, "March"); break;
					case 4: painter->drawText( day+5, monthY, "April"); break;
					case 5: painter->drawText( day+5, monthY, "May"); break;
					case 6: painter->drawText( day+5, monthY, "June"); break;
					case 7: painter->drawText( day+5, monthY, "July"); break;
					case 8: painter->drawText( day+5, monthY, "August"); break;
					case 9: painter->drawText( day+5, monthY, "September"); break;
					case 10: painter->drawText( day+5, monthY, "October"); break;
					case 11: painter->drawText( day+5, monthY, "November"); break;
					case 12: painter->drawText( day+5, monthY, "December"); break;
					}
				}
			if( tempdate == tempdate.currentDate() )    // if current date
				{
				painter->setPen( "red" );
				painter->drawLine( day, TEXTPIXY+1, day, pageheight );  // Draws a solid red line for the current date
				if ( weekdetail)
					{
					painter->setBrush ( "pink" );
					painter->drawRect( day, TEXTPIXY+1, daypix, pageheight);
					}
				}
			tempdate = tempdate.addDays(1);
			}
		
		// Draws the tasks, mains and milestones
		for( i; i<project->numtasks && page==currentpage; i++ )  // for each task
			{
//printf("top of loop i is %d, page is %d, currentpage is %d, i%%pagetasks is %d\n", i, page, currentpage, i%pagetasks);

			// Switch to a new page, if we've filled this one (only
			// when printing)
			if( i!=0 && (i%pagetasks) == 0 )
				{
//printf("new page\n");
				if ( isprinting )
					printer->newPage();
				page++;
				}

			if( page == currentpage && project->tasks[i] )   // if this tasknumber is read from file
				{
//printf("drawing task\n");
				int j=0;
				if( project->tasks[i]->after != 0 )	// if 'after' is specified (which means the task depends on other tasks)
					{			// search for the last other task
					while( project->tasks[j]->num !=
project->tasks[i]->after && j < i )
						{ j++; }
					}
				
//printf("i is %d, j is %d\n", i, j);
				int jbegin, jend;
				if( (int)( (float)(i/pagetasks)-(float)(j/pagetasks) ) > 0 )
					{
					jbegin = 0;
					jend = 1200;
					}
				else
					{
					jbegin = j%pagetasks;
					jend = j%pagetasks;
					}
//printf("jbegin is %d, jend is %d\n", jbegin, jend);
				
				if( project->tasks[i]->type == TASK )   // draws a normal task
					{
					painttask( painter, i, i%pagetasks );
					if( project->tasks[i]->after != 0 )                     // if 'after' is specified (which means the task depends on other tasks)
						{ painttaskendarrow( painter, i, i%pagetasks, jend ); }   // draws first part of the dependency arrow
					}
				else if( project->tasks[i]->type == MAIN )   // draws a main task
					{
					paintmain( painter, i, i%pagetasks );
					if( project->tasks[i]->after != 0 )                     // if 'after' is specified (which means the task depends on other tasks)
						{ paintmainendarrow( painter, i, i%pagetasks, jend ); }   // draws first part of the dependency arrow
					}
				else if( project->tasks[i]->type == MILE )  // draws a milestone
					{
					paintmile( painter, i, i%pagetasks );
					if( project->tasks[i]->after != 0 )                     // if 'after' is specified (which means the task depends on other tasks)
						{ paintmileendarrow( painter, i, i%pagetasks, jend ); }   // draws first part of the dependency arrow
					}
				
				if( project->tasks[j]->type == TASK && 
				    project->tasks[i]->after != 0 )   // draws a normal task
					{
					painttaskbeginarrow( painter, i, i%pagetasks, jbegin );   // draws first part of the dependency arrow
					}
				else if( project->tasks[j]->type == MAIN && 
					 project->tasks[i]->after != 0 )   // draws a main task
					{
					paintmainbeginarrow( painter, i, i%pagetasks, jbegin );   // draws first part of the dependency arrow
					}
				else if( project->tasks[j]->type == MILE && 
					 project->tasks[i]->after != 0 )  // draws a milestone
					{
					paintmilebeginarrow( painter, i, i%pagetasks, jbegin );   // draws first part of the dependency arrow
					}
				}
//printf("drawing grid\n");

			// draw the grid line
			painter->setPen( "darkgrey" );
			painter->drawLine( 0, TEXTPIXY+((i+1)%pagetasks)*SPACING, pagewidth, TEXTPIXY+((i+1)%pagetasks)*SPACING );


			}
		}
	}


void Gantt::paintEvent( QPaintEvent *e )  // repaints automagically when needed
	{
	draw();
	}


void Gantt::painttask( QPainter *painter, int taskno, int y )
	{
	int taskX, taskY, taskW;
	painter->setPen( "black" );
	painter->setBrush( "lightblue" );

	// Write description
	painter->setFont( QFont("helvetica", FONTSIZE, QFont::Normal) );
	painter->drawText( 10, TEXTPIXY+13+y*SPACING,
	                   textpixx-10, 20, AlignLeft,
	                   project->tasks[taskno]->description->copy() );

	// Draw rectangle; take percentage completed into account
	taskX = textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0));
	taskY = TEXTPIXY+10+y*SPACING;
	// this is the full width
	taskW = daypix * project->tasks[taskno]->startdate->daysTo( project->tasks[taskno]->enddate->addDays(0));
	painter->drawRect( taskX, taskY, taskW, 15);
	if ( project->tasks[taskno]->completed > 0)
		{
		painter->setBrush( "blue" );
		painter->setPen( "blue" );
		int taskWcomp = (taskW * project->tasks[taskno]->completed) / 100;
		painter->drawRect( taskX+1, taskY+1, taskWcomp-2, 13);
		}
	}


void Gantt::paintmain( QPainter *painter, int taskno, int y )
	{
	painter->setPen( "black" );
	painter->setBrush( "black" );
	painter->setFont( QFont("helvetica", FONTSIZE, QFont::Bold) );
	painter->drawText( 10, TEXTPIXY+13+y*SPACING,
	                   textpixx-10, 20, AlignLeft,
	                   project->tasks[taskno]->description->copy() );
	QPointArray a;
	a.setPoints( 8,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ) - 5,
	             TEXTPIXY+15+y*SPACING,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->enddate->addDays(0) ) + 5,
	             TEXTPIXY+15+y*SPACING,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->enddate->addDays(0) ) + 5,
	             TEXTPIXY+20+y*SPACING,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->enddate->addDays(0) ),
	             TEXTPIXY+25+y*SPACING,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->enddate->addDays(0) ) - 5,
	             TEXTPIXY+20+y*SPACING,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ) + 5,
	             TEXTPIXY+20+y*SPACING,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
	             TEXTPIXY+25+y*SPACING,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ) - 5,
	             TEXTPIXY+20+y*SPACING );
	painter->drawPolygon( a );
	}


void Gantt::paintmile( QPainter *painter, int taskno, int y )
	{
	painter->setPen( "black" );
	painter->setBrush( "black" );
	painter->setFont( QFont("helvetica", FONTSIZE, QFont::Normal, 1 ) );
	painter->drawText( 10, TEXTPIXY+13+y*SPACING,
	                   textpixx-10, 20, AlignLeft,
	                   project->tasks[taskno]->description->copy() );
	QPointArray a;
	a.setPoints( 4,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
	             TEXTPIXY+15+y*SPACING,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ) + 5,
	             TEXTPIXY+20+y*SPACING,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
	             TEXTPIXY+25+y*SPACING,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ) - 5,
	             TEXTPIXY+20+y*SPACING );
	painter->drawPolygon( a );
	QString qstr;
	painter->setFont( QFont("helvetica", FONTSIZE, QFont::Bold) );
	painter->drawText( textpixx + daypix * project->startdate->daysTo(
project->tasks[taskno]->startdate->addDays(0) ) + 10,
	                   TEXTPIXY+25+y*SPACING,
	                   qstr.setNum( project->tasks[taskno]->startdate->day()
) + "." + qstr.setNum( project->tasks[taskno]->startdate->month() ) );
	}


void Gantt::painttaskbeginarrow( QPainter *painter, int taskno, int y, int j )
	{
	if ( project->tasks[j]->enddate->daysTo( project->tasks[taskno]->startdate->addDays(0) ) * daypix < 7 )
		{
		painter->drawLine( textpixx + daypix * project->startdate->daysTo( project->tasks[j]->enddate->addDays(0) ),
		                   TEXTPIXY+17+j*SPACING,
		                   textpixx + daypix * project->startdate->daysTo( project->tasks[j]->enddate->addDays(0) ) + 7,
		                   TEXTPIXY+17+j*SPACING );
		painter->drawLine( textpixx + daypix * project->startdate->daysTo( project->tasks[j]->enddate->addDays(0) ) + 7,
		                   TEXTPIXY+17+j*SPACING,
		                   textpixx + daypix * project->startdate->daysTo( project->tasks[j]->enddate->addDays(0) ) + 7,
		                   TEXTPIXY+27+j*SPACING );
		painter->drawLine( textpixx + daypix * project->startdate->daysTo( project->tasks[j]->enddate->addDays(0) ) + 7,
		                   TEXTPIXY+27+j*SPACING,
		                   textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+27+j*SPACING );
		}
	else
		{
		painter->drawLine( textpixx + daypix * project->startdate->daysTo( project->tasks[j]->enddate->addDays(0) ),
		                   TEXTPIXY+17+j*SPACING,
		                   textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+17+j*SPACING );
		painter->drawLine( textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+17+j*SPACING,
		                   textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+27+j*SPACING );
		}
	}


void Gantt::painttaskendarrow( QPainter *painter, int taskno, int y, int j )
	{
	painter->setPen( "black" );
	painter->setBrush( "black" );
	painter->drawLine( textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
	                   TEXTPIXY+27+j*SPACING,
	                   textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
	                   TEXTPIXY+3+y*SPACING );
	QPointArray a;
	a.setPoints( 3,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ) - 5,
	             TEXTPIXY+4+y*SPACING,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ) + 5,
	             TEXTPIXY+4+y*SPACING,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
	             TEXTPIXY+9+y*SPACING );
	painter->drawPolygon( a );
	}


void Gantt::paintmainbeginarrow( QPainter *painter, int taskno, int y, int j )
	{
	if ( project->tasks[j]->enddate->daysTo( project->tasks[taskno]->startdate->addDays(0) ) * daypix < 7 )
		{
		painter->drawLine( textpixx + daypix * project->startdate->daysTo( project->tasks[j]->enddate->addDays(0) ),
		                   TEXTPIXY+26+j*SPACING,
		                   textpixx + daypix * project->startdate->daysTo( project->tasks[j]->enddate->addDays(0) ),
		                   TEXTPIXY+27+j*SPACING );
		painter->drawLine( textpixx + daypix * project->startdate->daysTo( project->tasks[j]->enddate->addDays(0) ),
		                   TEXTPIXY+27+j*SPACING,
		                   textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+27+j*SPACING );
		}
	else
		{
		painter->drawLine( textpixx + daypix * project->startdate->daysTo( project->tasks[j]->enddate->addDays(0) ),
		                   TEXTPIXY+18+j*SPACING,
		                   textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+18+j*SPACING );
		painter->drawLine( textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+18+j*SPACING,
		                   textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+27+j*SPACING );
		}
	}


void Gantt::paintmainendarrow( QPainter *painter, int taskno, int y, int j )
	{
	painter->setPen( "black" );
	painter->setBrush( "black" );
	painter->drawLine( textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
	                   TEXTPIXY+27+j*SPACING,
	                   textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
	                   TEXTPIXY+8+y*SPACING );
	QPointArray a;
	a.setPoints( 3,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ) - 5,
	             TEXTPIXY+9+y*SPACING,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ) + 5,
	             TEXTPIXY+9+y*SPACING,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
	             TEXTPIXY+14+y*SPACING );
	painter->drawPolygon( a );
	}


void Gantt::paintmilebeginarrow( QPainter *painter, int taskno, int y, int j )
	{
	if ( project->tasks[j]->enddate->daysTo( project->tasks[taskno]->startdate->addDays(0) ) * daypix < 7 )
		{
		painter->drawLine( textpixx + daypix * project->startdate->daysTo( project->tasks[j]->enddate->addDays(0) ),
		                   TEXTPIXY+20+j*SPACING,
		                   textpixx + daypix * project->startdate->daysTo( project->tasks[j]->enddate->addDays(0) ) + 7,
		                   TEXTPIXY+20+j*SPACING );
		painter->drawLine( textpixx + daypix * project->startdate->daysTo( project->tasks[j]->enddate->addDays(0) ) + 7,
		                   TEXTPIXY+20+j*SPACING,
		                   textpixx + daypix * project->startdate->daysTo( project->tasks[j]->enddate->addDays(0) ) + 7,
		                   TEXTPIXY+27+j*SPACING );
		painter->drawLine( textpixx + daypix * project->startdate->daysTo( project->tasks[j]->enddate->addDays(0) ) + 7,
		                   TEXTPIXY+27+j*SPACING,
		                   textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+27+j*SPACING );
		}
	else
		{
		painter->drawLine( textpixx + daypix * project->startdate->daysTo( project->tasks[j]->enddate->addDays(0) ),
		                   TEXTPIXY+20+j*SPACING,
		                   textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+20+j*SPACING );
		painter->drawLine( textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+20+j*SPACING,
		                   textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
		                   TEXTPIXY+27+j*SPACING );
		}
	}


void Gantt::paintmileendarrow( QPainter *painter, int taskno, int y, int j )
	{
	painter->setPen( "black" );
	painter->setBrush( "black" );
	painter->drawLine( textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
	                   TEXTPIXY+27+j*SPACING,
	                   textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
	                   TEXTPIXY+7+y*SPACING );
	QPointArray a;
	a.setPoints( 3,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ) - 5,
	             TEXTPIXY+8+y*SPACING,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ) + 5,
	             TEXTPIXY+8+y*SPACING,
	             textpixx + daypix * project->startdate->daysTo( project->tasks[taskno]->startdate->addDays(0) ),
	             TEXTPIXY+13+y*SPACING );
	painter->drawPolygon( a );
	}
