//===========================================================================
//
//  Filename  : main.h
//  Purpose   : Application and Controller for QtGantt
//  Copyright : Copyright (C) 2000 Matt J. Gumbley <matt@gumbley.demon.co.uk>
//  Original  : by Gunnstein Lye
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//===========================================================================

#ifndef MAIN_H
#define MAIN_H

#include <qapplication.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qwidget.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qscrollbar.h>
#include <qspinbox.h>
#include <qaccel.h>
#include <qfile.h>

#include "gantt.h"
#include "project.h"

#define MINDAYPIX 1
#define MAXDAYPIX 40

class MainWidget : public QWidget
	{
	Q_OBJECT	// because MainWidget contains slots
	public:
		MainWidget( char *filenamearg, QWidget *parent=0, const char *name=0 );
	protected:
		void resizeEvent( QResizeEvent * );
	private:
		QFrame *frame;
		Project *project;	// This is the model
		Gantt *gantt;		// This is one view (there will be more)
		QScrollBar *scrolltextpix;
		QSpinBox *spindaypix;
		QAccel *f5acc;
		QAccel *plusacc;
		QAccel *minusacc;
		QMenuBar *_menubar;
		QPopupMenu *_filemenu;
		QPopupMenu *_viewmenu;
		QPopupMenu *_options;
		QPopupMenu *_helpmenu;
		int weekdetailID;
		bool isWeekdetail;
		int frameT, frameY, frameW; // unimportant positioning stuff
		// frameT is the thickness of the frame; frameY is where it is
		// positioned vertically

	private slots:
		void slotNewFile();
		void slotOpenFile();
		void slotSaveFile();
		void slotRefreshFile();
		void slotPrintFile();

		void slotQuit();
		void slotAbout();
		void slottoggleweekdetail();
		void slotzoomIn();
		void slotzoomOut();
	};

#endif // MAIN_H
