//===========================================================================
//
//  Filename  : project.cpp
//  Purpose   : Project Model for QtGantt
//  Copyright : Copyright (C) 2000 Matt J. Gumbley <matt@gumbley.demon.co.uk>
//  Original  : by Gunnstein Lye
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//===========================================================================

#include <iostream.h>

#include <qfile.h>
#include <qdatetime.h>
#include <qstring.h>
#include <qtextstream.h>

#include "task.h"
#include "project.h"

Project::Project( char *filename )
{
	int t;
	for (t=0; t<NUMTASKS; t++)
	{
		tasks[t] = 0;
	}
	name = 0;
	startdate = 0;
	numtasks = 0;
	changed = 0;
	this->filename = 0;
	if (filename)
		openFilename(filename);
}

Project::~Project()
{
	reset();
	// any more deallocation to be done here?
}

void Project::openFilename( const char *filename )
{
	reset();
	QFile * file = new QFile();
	this->filename = new QString(filename);
	file->setName( filename );
	if (file->exists() )
		{
		open( file );
		}
	else
		{
		printf("file %s doesn't exist\n", filename);
		}
}

void Project::refresh()
	{
	// Can't do much if there's no project loaded!
	if (this->filename == 0) {
		fprintf(stderr,"project filename is null - not loaded, so can't refresh!\n");
		return;
	}
	QString oldfilename = this->filename->copy();
	reset();
	this->filename = new QString( oldfilename );
	QFile * file = new QFile();
	file->setName( (const char *) oldfilename );
	if( file->exists() )
		{
		open( file );
		}
	}


bool Project::isChanged()
{
	return changed;
}

void Project::reset()
{
	int t;
	for (t=0; t<numtasks; t++)
	{
		if (tasks[t]) {
			delete tasks[t];
			tasks[t] = 0;
		}
	}
	if (name) {
		delete name;
		name = 0;
	}
	if (startdate) {
		delete startdate;
		startdate = 0;
	}
	// delete tasks;
	numtasks = 0;
	changed = 0;
	if (filename) {
		delete filename;
		filename = 0;
	}
}

void Project::open( QFile *file )
	{
	if( file->open(IO_ReadOnly) )
		{
		QTextStream t( file );
		read( &t );
		file->close();
		}
	else
		{
		printf( "File not found\n" );
		}
	}


void Project::save( )
{
}

void Project::read( QTextStream *t )
{
	QString qstr1, qstr2, yyyy, mm, dd;
	numtasks=0;
	int pos1, pos2, pos3;
	
	while ( !t->eof() && numtasks<NUMTASKS )     // loops through the file
		{
		qstr1 = t->readLine();
		
		if( !qstr1.isEmpty() && qstr1.left(1) != "#" )  // if not an empty line and not a comment
			{
			if( qstr1.left(1) == "%" )    // if a header line
				{
				pos1 = qstr1.find( '%', 1, TRUE );
				qstr2 = qstr1.mid( 1, pos1-1 );
				
				pos2 = qstr2.find( '-', 0, TRUE );
				dd = qstr2.left(pos2);
				pos3 = qstr2.find( '-', pos2+1, TRUE );
				mm = qstr2.mid( pos2+1, pos3-pos2-1 );
				yyyy = qstr2.right( qstr2.length() - pos3-1 );
				startdate = new QDate( yyyy.toInt(), mm.toInt(), dd.toInt() );
				
				qstr2 = qstr1.right( qstr1.length() - pos1 - 1 );
				name = new QString( qstr2 );
				}
			else                   // if a task line
				{
				tasks[numtasks] = new Task();
				// num
				pos1 = qstr1.find( ':', 0, TRUE );
				qstr2 = qstr1.left(pos1);
				tasks[numtasks]->num = qstr2.toInt();
				
				// type
				pos2 = qstr1.find( ':', pos1+1, TRUE );
				qstr2 = qstr1.mid( pos1+1, pos2-pos1-1 );
				if ( qstr2 == "main" )
					tasks[numtasks]->type = MAIN;
				else if ( qstr2 == "task" )
					tasks[numtasks]->type = TASK;
				else if ( qstr2 == "mile" )
					tasks[numtasks]->type = MILE;
				else
					{
						printf("Unknown task type '%s'\n",&qstr2);
					}
				
				// startdate
				pos1 = qstr1.find( ':', pos2+1, TRUE );
				qstr2 = qstr1.mid( pos2+1, pos1-pos2-1 );
				pos3 = qstr2.find( '-', 0, TRUE );
				dd = qstr2.left(pos3);
				pos2 = qstr2.find( '-', pos3+1, TRUE );
				mm = qstr2.mid( pos3+1, pos2-pos3-1 );
				yyyy = qstr2.right( qstr2.length() - pos2-1 );
				tasks[numtasks]->startdate = new QDate( yyyy.toInt(), mm.toInt(), dd.toInt() );
				
				// enddate
				pos2 = qstr1.find( ':', pos1+1, TRUE );
				if( tasks[numtasks]->type != MILE )
					// if not a milestone
					{
					qstr2 = qstr1.mid( pos1+1, pos2-pos1-1 );
					pos3 = qstr2.find( '-', 0, TRUE );
					dd = qstr2.left(pos3);
					pos1 = qstr2.find( '-', pos3+1, TRUE );
					mm = qstr2.mid( pos3+1, pos1-pos3-1 );
					yyyy = qstr2.right( qstr2.length() - pos1-1 );
					tasks[numtasks]->enddate = new QDate( yyyy.toInt(), mm.toInt(), dd.toInt() );
					}
				else         // milestones don't need enddate, so a blank entry is ok
					tasks[numtasks]->enddate = new QDate( tasks[numtasks]->startdate->addDays(0) );
				
				// after
				pos1 = qstr1.find( ':', pos2+1, TRUE );
				qstr2 = qstr1.mid( pos2+1, pos1-pos2-1 );
				tasks[numtasks]->after = qstr2.toInt();
				
				// percentage copleted
				pos2 = qstr1.find( ':', pos1+1, TRUE );
				qstr2 = qstr1.mid( pos1+1, pos2-pos1-1 );
				tasks[numtasks]->completed = qstr2.toInt();
				
				// description
				qstr2 = qstr1.right( qstr1.length() - pos2-1 );
				tasks[numtasks]->description = new QString( qstr2 );
				
				numtasks++;
				}
			}
		}
}

void Project::write( QTextStream *t )
{
}

